$title  A Transportation Problem (TRNSPORT,SEQ=1)
$onUNDF
$onText
This problem finds a least cost shipping schedule that meets
requirements at markets and supplies at factories.


Dantzig, G B, Chapter 3.3. In Linear Programming and Extensions.
Princeton University Press, Princeton, New Jersey, 1963.

This formulation is described in detail in:
Rosenthal, R E, Chapter 2: A GAMS Tutorial. In GAMS: A User's Guide.
The Scientific Press, Redwood City, California, 1988.

The line numbers will not match those in the book because of these
comments.

Keywords: linear programming, transportation problem, scheduling
$offText

Set
   i 'canning plants' / seattle,  san-diego /
   j 'markets'        / new-york, chicago, topeka /;

Parameter
   a(i) 'capacity of plant i in cases'
        / seattle     350
          san-diego   600  /

   b(j) 'demand at market j in cases'
        / new-york    325
          chicago     300
          topeka      275  /;

Table d(i,j) 'distance in thousands of miles'
               new-york       chicago        topeka
   seattle          2.5           1.7           1.8
   san-diego        2.5           1.8           1.4;

Scalar f 'freight in dollars per case per thousand miles' / 90 /;

Parameter c(i,j) 'transport cost in thousands of dollars per case';
c(i,j) = f * d(i,j) / 1000;

Variable
   x(i,j) 'shipment quantities in cases'
   z      'total transportation costs in thousands of dollars';

Positive Variable x;

Equation
   cost      'define objective function'
   supply(i) 'observe supply limit at plant i'
   demand(j) 'satisfy demand at market j';

cost..        z  =e=  sum((i,j), c(i,j)*x(i,j));

supply(i)..   sum(j, x(i,j)) =l= a(i);

demand(j)..   sum(i, x(i,j)) =g= b(j);

Model mod / all /;

set allcases /base,case1*case3/;
set cases(allcases) /#allcases/;
set sd /supply,demand/;
set ht / modelStat, solveStat, resUsd /;
table changetomake(allcases,sd) data to change by case - note make sure to keep supply ge than demand
             supply  demand
base
case1          1.1
case2          1.05    1.01
case3                  0.90;
parameter  saved_a(i),
           saved_b(j),
           saved_x_solution(i,j,allcases)
           saved_supply_shadow_price(i,allcases)
           saved_demand_shadow_price(j,allcases)
           saved_solution_stuff(*,allcases)
           mInfo(ht)
           supplychange
           demandchange;

           saved_a(i)=a(i);
           saved_b(j)=b(j);

option solvelink=2;
$set useSolverLog 1
$set solverlog
$if set useSolverLog $set solverlog output=sys.stdout

embeddedCode Python:
#make available a function -  gmoGetHeadnTail - that allows recovery of model attributes
from gmomcc import *
#identify a function which causes a solve of the model instance named mi
# in doing the solve the data in te symIn list Will be changed
# and the data in the symOut list plus that in mInfo will be collected from the solver and put back into gams
def solveMI(mi, mInfo, symIn=[], symOut=[]):
#move data from gams into the data within a model instance
# note db is a class Within the Python API and is desribed at https://www.gams.com/latest/docs/apis/python/classgams_1_1database_1_1GamsDatabase.html
  for sym in symIn:
    gams.db[sym].copy_symbol(mi.sync_db[sym])
#solve the model instance
  mi.solve(%solverlog%)
#bring solution  back into GAMS from the solver
  for sym in symOut:
    mi.sync_db[sym].copy_symbol(gams.db[sym])
#gather up information on solution status this into a parameter
  gams.set(mInfo,[('modelStat',mi.model_status), ('solveStat',mi.solver_status), ('resUsd', gmoGetHeadnTail(mi._gmo,gmoHresused))])
pauseEmbeddedCode

*do initial solve so that model instance is present
*a and b at ened are what data to change
* .zero is what to do with no data on a and it adds a zero
$libinclude pyEmbMI miTran 'mod using lp min z' -all_model_types=cplexd a.Zero b.Zero

loop(cases,
    supplychange=1;
    supplychange $changetomake(cases,"supply")= changetomake(cases,"supply");
    demandchange=1;
    demandchange $changetomake(cases,"demand")= changetomake(cases,"demand");
    a(i)=saved_a(i)*supplychange;
    b(j)=saved_b(j)*demandchange;
*   solve transport using lp minimizing z;
    option solPrint=silent;
    continueEmbeddedCode:
    solveMI(miTran,'mInfo',['a','b'],['z','x','supply','demand'])
    pauseEmbeddedCode z x supply demand mInfo
    saved_x_solution(i,j,cases)=x.l(i,j);
    saved_supply_shadow_price(i,cases)=supply.m(i);
    saved_demand_shadow_price(j,cases)=demand.m(j);
    saved_solution_stuff("Obj Value",cases)=z.l;
    saved_solution_stuff(ht,cases)=mInfo(ht);
    saved_solution_stuff("Number_infeasibilities",cases)=sum(i$supply.infeas(i),1) +
                                                         sum(j$demand.infeas(j),1) +
                                                         sum((i,j)$x.infeas(i,j),1);
);
display saved_solution_stuff;

